<div class="panel panel-default">
    <div class="panel-heading">Agent Status
    </div>
    <div class="panel-body">

        <table class="table">
            <thead>

            <tr>
                <th></th>
                <th>Added / Updated Events</th>
                <th>Updated Alarm Flags</th>
                <th>Removed Events</th>
                <th>Last Activity</th>
            </tr>

            </thead>

            <tbody>

            <!-- EventDB Value Updater -->
            <tr>
                <th>Import API</th>
                <td>{{item.addedEvents}}</td>
                <td>{{item.updatedAlarmFlags}}</td>
                <td>{{item.removedEvents}}</td>
                <td>{{item.lastActivity}}</td>

            </tr>

            </tbody>
        </table>
    </div>
</div>
